/* misc.h -- header for miscellaneous library-like routines in fsgrab
 *
 * Aaron Crane <aaronc@pobox.com>
 *
 * This file is part of `e2recover', a suite of programs which assist in
 * recovering deleted files from ext2 file systems.  `e2recover' is
 * Copyright  1997, 1998, 1999 Aaron Crane <aaronc@pobox.com>.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#if HAVE_CONFIG_H
#include <config.h>
#endif

#include <stddef.h>
#include <stdlib.h>
#include <sys/types.h>
#include <stdio.h>

extern const char *invocation_name;

#ifndef __GNUC__
#define __attribute__(attrs)
#endif

void message (const char *fmt, ...) __attribute__ ((format (printf, 1, 2)));
void /*@exits@*/ fatal (const char *fmt, ...) __attribute__ ((format (printf, 1, 2), noreturn));
void /*@exits@*/ fataln (int status, const char *fmt, ...) __attribute__ ((format (printf, 2, 3), noreturn));
void *mustmalloc (size_t n);
size_t safe_atosz (const char *s, int allow_multiplier);

void /*@exits@*/ short_usage (void) __attribute__ ((noreturn));


#if HAVE_LIBEXT2FS

#if HAVE_EXT2FS_EXT2FS_H
/* <ext2fs/ext2fs.h> uses these structs without even a forward declaration
 * for them, which causes warnings for me because I don't include
 * <linux/ext2_fs.h>. */
struct ext2_inode; struct ext2_dir_entry;

/* <ext2fs/ext2fs.h> has a bunch of inline functions which dereference
 * pointers to structs which aren't defined if you don't include kernel
 * files.  It lets you define this bletcherous token to disable that
 * breakage.
 */
#define NO_INLINE_FUNCS

#include <ext2fs/ext2fs.h>
#endif /* HAVE_EXT2FS_EXT2FS_H */

#define SEEK_FUNC ext2fs_llseek
typedef ext2_loff_t my_offset_t;


#elif HAVE_LLSEEK

#define SEEK_FUNC llseek
typedef loff_t my_offset_t;


#else /* Neither ext2fs_llseek nor llseek */
#define SEEK_FUNC lseek
typedef off_t my_offset_t;
#endif

#define SEEK(fd, offset, whence) SEEK_FUNC (fd, offset, whence)
#define STR(x) #x
#define SEEKNAME STR(SEEK_FUNC)
